/*

fiber.c - Fibers

*/

/*...sincludes:0:*/
#include <stdlib.h>
#include "types.h"
#include "interrupts.h"
#include "list.h"
#include "fiber.h"
#include "log.h"

/*...vtypes\46\h:0:*/
/*...vinterrupts\46\h:0:*/
/*...vlist\46\h:0:*/
/*...vfiber\46\h:0:*/
/*...e*/

static LIST fiber_list;
static BOOLEAN fiber_draining = FALSE;

/*...sfiber_init:0:*/
void fiber_init(void)
	{
	list_init(&fiber_list);
	}
/*...e*/
void fiber_show(void)
    {
    FIBER *f = list_first(&fiber_list);
    while (f)
        {
        log_printf ("f = $");
        log_word ((word) f);
        log_printf (", nf = $");
        log_word ((word) f->nf);
        log_printf (", nfp = $");
        log_word ((word) f->nfp);
        log_printf ("\r\n$");
        f = list_next (f);
        }
    }
/*...sfiber_enqueue:0:*/
void fiber_enqueue(FIBER *f)
	{
#ifdef DEBUG
    log_printf ("fiber_enqueue:\r\n$");
#endif
    if ( ! f->queued )
        {
        interrupts_disable();
        f->queued = 1;
        list_add_last(&fiber_list, f);
#ifdef DEBUG
        fiber_show();
#endif
        interrupts_enable();
        }
#ifdef DEBUG
    else
        {
        log_word ((word) f);
        log_printf (" already queued\r\n$");
        }
#endif
	}
/*...e*/
void fiber_dequeue(FIBER *f)
	{
#ifdef DEBUG
    log_printf ("fiber_dequeue:\r\n$");
#endif
    if ( f->queued )
        {
        interrupts_disable();
        f->queued = 0;
#ifdef DEBUG
        if ( ! list_remove_item(&fiber_list, f) )
            {
            log_word ((word) f);
            log_printf (" not found\r\n$");
            }
#else
        list_remove_item(&fiber_list, f);
#endif
        fiber_show();
        interrupts_enable();
        }
#ifdef DEBUG
    else
        {
        log_word ((word) f);
        log_printf (" not queued\r\n$");
        }
#endif
	}

/*...sfiber_drainer:0:*/
static void fiber_drainer(void)
	{
	FIBER *f;
	fiber_draining = TRUE;
    // WJB: Changed "while" to "if"
	if ( (f = list_remove_first(&fiber_list)) != NULL )
		{
		interrupts_enable();
#ifdef DEBUG
        log_printf ("fiber_dispatch: $");
        log_word ((word) f->nf);
        log_printf (" ($");
        log_word ((word) f->nfp);
        log_printf (")\r\n$");
        fiber_show();
#endif
        f->queued = 0;
		f->nf(f->nfp);
		interrupts_disable();
		}
	fiber_draining = FALSE;
	}
/*...e*/
/*...sfiber_drain:0:*/
void fiber_drain(void)
	{
	interrupts_disable();
	fiber_drainer();
	interrupts_enable();
	}
/*...e*/
/*...sfiber_enqueue_isr:0:*/
void fiber_enqueue_isr(FIBER *f)
	{
	list_add_last(&fiber_list, f);
	}
/*...e*/
/*...sfiber_drain_isr:0:*/
void fiber_drain_isr(void)
	{
	if ( !fiber_draining )
		fiber_drainer();
	}
/*...e*/
